import 'package:meta/meta.dart';

/// Calculate [Duration] in fractional hours.
/// 
/// A [Duration]'s `inHours` method does not measure out 
/// fractions of an hour. This method converts [d] in minutes 
/// to hours manually, thus preserving the fractions of hours
/// that may be in each one.
double durationExactHours( Duration d )
{
	return ( (d.inSeconds) / 60 ) / 60;
}

/// Determines whether or not there are more of the [smallerUnit] than 
/// the [biggerUnit].
bool _isMoreOfSmallerUnit( int smallerUnit, int biggerUnit )
{
	return _shouldMakePlural(smallerUnit - (biggerUnit * 60));
}

/// Determines whether [initialString] should have an "s" at the end of it.
///
/// This is based on whether or not [multiple] is `true` or `false`.
String _makeTimeUnitPlural( String initialString, {@required bool multiple} )
{
	if ( multiple )
	{
		initialString += 's';
	}
	return initialString;
}

/// According to the english language, quantities
/// other than one should be plural.
///
/// In accordance with this, test [unitValue] to see
/// if this rule should be applied.
bool _shouldMakePlural( num unitValue )
{
	return unitValue != 1;
}

/// Return a human-readable version of [Duration] [d].
String durationHumanReadable( Duration d )
{
	final List<String> _durationSplit = d.toString().split(':');

	final String hourString	= _makeTimeUnitPlural( 
			'hour', 
			multiple: _shouldMakePlural(
					durationExactHours(d)
				)
		);
	final String minuteString = _makeTimeUnitPlural(
			'minute',
			multiple: _isMoreOfSmallerUnit(d.inMinutes, d.inHours)
		);
	final String secondString = _makeTimeUnitPlural(
			'second',
			multiple: _isMoreOfSmallerUnit(d.inSeconds, d.inMinutes)
		);

	return '${_durationSplit[0]} $hourString, '
			'${_durationSplit[1]} $minuteString, '
			'${_durationSplit[2].substring(0,2)} $secondString';
}
